@@ -2,4 +2,4 @@ if (getRversion() >= '2.15.1') globalVariables(c('buttonsFrame',
  'dcFrame', 'dcVariable', 'distFrame', 'distVariable', 'formulaFrame',
  'methodtypeFrame', 'methodtypeVariable', 'outerOperatorsFrame', 
  'rhsVariable', 'subsetFrame', 'subsetVariable', 'top', 'covariatesBox',
- 'covariatesFrame'))
+ 'covariatesFrame', 'bidTable', 'responseFrame', 'currencyBox'))
